@@ -13,14 +13,14 @@
 # limitations under the License.
 
 import multiprocessing
-import os
 import optparse
+import os
 import re
-import sys
 
-from event_log import EventLog
-from error import NoSuchProjectError
 from error import InvalidProjectGroupsError
+from error import NoSuchProjectError
+from error import RepoExitError
+from event_log import EventLog
 import progress
 
 
@@ -42,6 +42,10 @@ WORKER_BATCH_SIZE = 32
 DEFAULT_LOCAL_JOBS = min(os.cpu_count(), 8)
 
 
+class UsageError(RepoExitError):
+    """Exception thrown with invalid command usage."""
+
+
 class Command(object):
     """Base class for any command line action in repo."""
 
@@ -71,7 +75,6 @@ class Command(object):
         repodir=None,
         client=None,
         manifest=None,
-        gitc_manifest=None,
         git_event_log=None,
         outer_client=None,
         outer_manifest=None,
@@ -80,7 +83,6 @@ class Command(object):
         self.client = client
         self.outer_client = outer_client or client
         self.manifest = manifest
-        self.gitc_manifest = gitc_manifest
         self.git_event_log = git_event_log
         self.outer_manifest = outer_manifest
 
@@ -215,7 +217,7 @@ class Command(object):
     def Usage(self):
         """Display usage and terminate."""
         self.OptionParser.print_usage()
-        sys.exit(1)
+        raise UsageError()
 
     def CommonValidateOptions(self, opt, args):
         """Validate common options."""
@@ -502,11 +504,5 @@ class MirrorSafeCommand(object):
     """
 
 
-class GitcAvailableCommand(object):
-    """Command that requires GITC to be available, but does not require the
-    local client to be a GITC client.
-    """
-
-
 class GitcClientCommand(object):
     """Command that requires the local client to be a GITC client."""
