@@ -15,7 +15,7 @@
 import contextlib
 import datetime
 import errno
-from http.client import HTTPException
+import http.client
 import json
 import os
 import re
@@ -26,11 +26,15 @@ from typing import Union
 import urllib.error
 import urllib.request
 
-from error import GitError, UploadError
+from error import GitError
+from error import UploadError
+from git_command import GitCommand
+from git_refs import R_CHANGES
+from git_refs import R_HEADS
+from git_refs import R_TAGS
 import platform_utils
 from repo_trace import Trace
-from git_command import GitCommand
-from git_refs import R_CHANGES, R_HEADS, R_TAGS
+
 
 # Prefix that is prepended to all the keys of SyncAnalysisState's data
 # that is saved in the config.
@@ -650,7 +654,7 @@ class Remote(object):
                     raise UploadError("%s: %s" % (self.review, str(e)))
                 except urllib.error.URLError as e:
                     raise UploadError("%s: %s" % (self.review, str(e)))
-                except HTTPException as e:
+                except http.client.HTTPException as e:
                     raise UploadError(
                         "%s: %s" % (self.review, e.__class__.__name__)
                     )
