@@ -21,6 +21,7 @@ which takes care of execing this entry point.
 """
 
 import getpass
+import json
 import netrc
 import optparse
 import os
@@ -31,35 +32,41 @@ import textwrap
 import time
 import urllib.request
 
+
 try:
     import kerberos
 except ImportError:
     kerberos = None
 
 from color import SetDefaultColoring
-import event_log
-from repo_trace import SetTrace, Trace, SetTraceToStderr
-from git_command import user_agent
-from git_config import RepoConfig
-from git_trace2_event_log import EventLog
 from command import InteractiveCommand
 from command import MirrorSafeCommand
-from command import GitcAvailableCommand, GitcClientCommand
-from subcmds.version import Version
 from editor import Editor
 from error import DownloadError
+from error import GitcUnsupportedError
 from error import InvalidProjectGroupsError
 from error import ManifestInvalidRevisionError
-from error import ManifestParseError
 from error import NoManifestException
 from error import NoSuchProjectError
 from error import RepoChangedException
-import gitc_utils
-from manifest_xml import GitcClient, RepoClient
-from pager import RunPager, TerminatePager
-from wrapper import WrapperPath, Wrapper
-
+from error import RepoError
+from error import RepoExitError
+from error import RepoUnhandledExceptionError
+from error import SilentRepoExitError
+import event_log
+from git_command import user_agent
+from git_config import RepoConfig
+from git_trace2_event_log import EventLog
+from manifest_xml import RepoClient
+from pager import RunPager
+from pager import TerminatePager
+from repo_trace import SetTrace
+from repo_trace import SetTraceToStderr
+from repo_trace import Trace
 from subcmds import all_commands
+from subcmds.version import Version
+from wrapper import Wrapper
+from wrapper import WrapperPath
 
 
 # NB: These do not need to be kept in sync with the repo launcher script.
@@ -97,6 +104,7 @@ else:
         )
 
 KEYBOARD_INTERRUPT_EXIT = 128 + signal.SIGINT
+MAX_PRINT_ERRORS = 5
 
 global_options = optparse.OptionParser(
     usage="repo [-p|--paginate|--no-pager] COMMAND [ARGS]",
@@ -299,11 +307,10 @@ class _Repo(object):
                 submanifest_path=gopts.submanifest_path,
                 outer_client=outer_client,
             )
-        gitc_manifest = None
-        gitc_client_name = gitc_utils.parse_clientdir(os.getcwd())
-        if gitc_client_name:
-            gitc_manifest = GitcClient(self.repodir, gitc_client_name)
-            repo_client.isGitcClient = True
+
+        if Wrapper().gitc_parse_clientdir(os.getcwd()):
+            print("GITC is not supported.", file=sys.stderr)
+            raise GitcUnsupportedError()
 
         try:
             cmd = self.commands[name](
@@ -312,7 +319,6 @@ class _Repo(object):
                 manifest=repo_client.manifest,
                 outer_client=outer_client,
                 outer_manifest=outer_client.manifest,
-                gitc_manifest=gitc_manifest,
                 git_event_log=git_trace2_event_log,
             )
         except KeyError:
@@ -331,20 +337,6 @@ class _Repo(object):
             )
             return 1
 
-        if (
-            isinstance(cmd, GitcAvailableCommand)
-            and not gitc_utils.get_gitc_manifest_dir()
-        ):
-            print(
-                "fatal: '%s' requires GITC to be available" % name,
-                file=sys.stderr,
-            )
-            return 1
-
-        if isinstance(cmd, GitcClientCommand) and not gitc_client_name:
-            print("fatal: '%s' requires a GITC client" % name, file=sys.stderr)
-            return 1
-
         try:
             copts, cargs = cmd.OptionParser.parse_args(argv)
             copts = cmd.ReadEnvironmentOptions(copts)
@@ -422,12 +414,36 @@ class _Repo(object):
             """
             try:
                 execute_command_helper()
-            except (KeyboardInterrupt, SystemExit, Exception) as e:
+            except (
+                KeyboardInterrupt,
+                SystemExit,
+                Exception,
+                RepoExitError,
+            ) as e:
                 ok = isinstance(e, SystemExit) and not e.code
+                exception_name = type(e).__name__
+                if isinstance(e, RepoUnhandledExceptionError):
+                    exception_name = type(e.error).__name__
+                if isinstance(e, RepoExitError):
+                    aggregated_errors = e.aggregate_errors or []
+                    for error in aggregated_errors:
+                        project = None
+                        if isinstance(error, RepoError):
+                            project = error.project
+                        error_info = json.dumps(
+                            {
+                                "ErrorType": type(error).__name__,
+                                "Project": project,
+                                "Message": str(error),
+                            }
+                        )
+                        git_trace2_event_log.ErrorEvent(
+                            f"AggregateExitError:{error_info}"
+                        )
                 if not ok:
-                    exception_name = type(e).__name__
                     git_trace2_event_log.ErrorEvent(
-                        f"RepoExitError:{exception_name}")
+                        f"RepoExitError:{exception_name}"
+                    )
                 raise
 
         try:
@@ -446,13 +462,13 @@ class _Repo(object):
                     "error: manifest missing or unreadable -- please run init",
                     file=sys.stderr,
                 )
-            result = 1
+            result = e.exit_code
         except NoSuchProjectError as e:
             if e.name:
                 print("error: project %s not found" % e.name, file=sys.stderr)
             else:
                 print("error: no project in current directory", file=sys.stderr)
-            result = 1
+            result = e.exit_code
         except InvalidProjectGroupsError as e:
             if e.name:
                 print(
@@ -466,7 +482,7 @@ class _Repo(object):
                     "the current directory",
                     file=sys.stderr,
                 )
-            result = 1
+            result = e.exit_code
         except SystemExit as e:
             if e.code:
                 result = e.code
@@ -474,6 +490,9 @@ class _Repo(object):
         except KeyboardInterrupt:
             result = KEYBOARD_INTERRUPT_EXIT
             raise
+        except RepoExitError as e:
+            result = e.exit_code
+            raise
         except Exception:
             result = 1
             raise
@@ -840,19 +859,30 @@ def _Main(argv):
             SetTraceToStderr()
 
         result = repo._Run(name, gopts, argv) or 0
+    except RepoExitError as e:
+        if not isinstance(e, SilentRepoExitError):
+            exception_name = type(e).__name__
+            print("fatal: %s" % e, file=sys.stderr)
+            if e.aggregate_errors:
+                print(f"{exception_name} Aggregate Errors")
+                for err in e.aggregate_errors[:MAX_PRINT_ERRORS]:
+                    print(err)
+            if (
+                e.aggregate_errors
+                and len(e.aggregate_errors) > MAX_PRINT_ERRORS
+            ):
+                diff = len(e.aggregate_errors) - MAX_PRINT_ERRORS
+                print(f"+{diff} additional errors ...")
+        result = e.exit_code
     except KeyboardInterrupt:
         print("aborted by user", file=sys.stderr)
         result = KEYBOARD_INTERRUPT_EXIT
-    except ManifestParseError as mpe:
-        print("fatal: %s" % mpe, file=sys.stderr)
-        result = 1
     except RepoChangedException as rce:
         # If repo changed, re-exec ourselves.
-        #
         argv = list(sys.argv)
         argv.extend(rce.extra_args)
         try:
-            os.execv(sys.executable, [__file__] + argv)
+            os.execv(sys.executable, [sys.executable, __file__] + argv)
         except OSError as e:
             print("fatal: cannot restart repo after upgrade", file=sys.stderr)
             print("fatal: %s" % e, file=sys.stderr)
