@@ -20,14 +20,15 @@ Temporary: Tracing is always on. Set `REPO_TRACE=0` to turn off.
 To also include trace outputs in stderr do `repo --trace_to_stderr ...`
 """
 
-import sys
+import contextlib
 import os
-import time
+import sys
 import tempfile
-from contextlib import ContextDecorator
+import time
 
 import platform_utils
 
+
 # Env var to implicitly turn on tracing.
 REPO_TRACE = "REPO_TRACE"
 
@@ -68,7 +69,7 @@ def _SetTraceFile(quiet):
     _TRACE_FILE = _GetTraceFile(quiet)
 
 
-class Trace(ContextDecorator):
+class Trace(contextlib.ContextDecorator):
     """Used to capture and save git traces."""
 
     def _time(self):
